# NOTE THIS NEEDS UPDATING

I have migrated this code to allow declaring the child scopes in child_scopes.json (in data/).

This allows creating a child scope without modifying any C++: just modify the child_scopes.json).

I need to update the docs!

What is written below (old docs) are mostly still valid (although incomplete) for the c++ approach.

# Making an aggregator scope from this template

This template simplifies the process of making an aggregator scope from a set of child scopes.

Each child scope is instantiated as a AggChildScope and its properties are defined with various class methods. Some are required, others are optional.

This is an overview. The source code is well commented.

These notes updated for bzr tag "click 0.4"

# Child scope ids

Each child scope has an id that consists of its fully qualified name. This is the click package name and its scope hook name separated by an underscore.

For example com.canonical.scopes.photos-local is a click package that contains a scope hook named 'photos-local'. Therefore its id is com.canonical.scopes.photos-local_photos-local.

The child scope id has to be entered in various places, including

* root CMakeLists.txt (define all scope ids as cmake vars)
* data/settings.ini.in.in (each child scope has a stanza including its cmake id var and other key bits)
* data/scope.ini.in.in (ensure the AggChildScopes key/value includes all child scope id vars from cmake) 
* include/scope-ids.h.in (needed to pull scope ids into c++ from cmake)

# Making AggChildScope instances

query.cpp has the ```make_child_scopes()``` method. This is where you create a AggChildScope instance for each child scope and fill in its basic properties. (See also aggchildscope.h for compehensive API docs)

For example

    AggChildScope bbc(BBC_ID, query_string);
    bbc.set_category_title(_("BBC"));
    bbc.set_cardinality(3);//override default set in childscopes.h
    bbc.set_surface_template(BBC);
    bbc.set_search_template(BBC);
    bbc.set_child_department("http://feeds.bbci.co.uk/news/technology/rss.xml");//display only this dept
    scopes_m[bbc.id()] = std::make_shared<AggChildScope>(bbc);
    scope_order.emplace_back(bbc.id());

See childscope.h for class/method docs.

## A couple key items

* Always make the shared pointer to the AggChildScope after running all configuration methods on it.
* The ```scope_order``` vector holds instantiated AggChildScope ids. It determines the order in which child scopes display in the aggregator scope.
* Later, the ```scopes``` vector is created to include all instantiated child scopes that are both enabled in settings and registered on the run time system.

# Settings enable/disable each scope

data/settings.ini.in.in allows the user to enable/disable any of the child scopes.

In query.cpp ```handle_enabled_child_scopes()``` method, only currently enabled (through user settings) child scopes are further defined and added to the vectors of current scopes.

# Handling query results

How to handle query results is defined in ```handle_child_scope_results()```.

Each categorised result arriving from each child scope has to be handled. This includes buffering it until all present & enabled child scopes that precede it have at least one result, thus ensuring the child scope categories display in the order we want. The first present & enabled child scope has its buffer bool set false (so it displays immediately), and the rest are set to true (so they wait until preceding child scopes have at least one child result). Each result therefore requires code to integrate it into the buffering system via a lambda function. This occurs by default for each child scope.

An ```if```/```if else```/```else``` construct allows you to provide special handling for child scope results.

If your child scope results needs no special handling you do not need to do anything. Such scopes are handled by the ```else``` expression.

If your child scope results do need special handling, create an 'if else (SCOPE ID)' expression based on the others. For example, you may need to do different things depending on whether the user has logged into the child scope through online accounts. When such a child scope is not logged in, the result's category id generally is set to reflect this. Thus you can check the arriving result's category id and respond appropriately. In the following example, the calls scope indicates when there are no results today with a category id of ```today_no_results`. This is checked, and different code paths follow.

    if (QString::fromStdString(res.category()->id()).startsWith("today_no_results")) {
        [ ...]
    } else {
        [ ... ]
    }

Regards,
Kyle Nitzsche <kyle.nitzsche@canonical.com>
11 March 2015
