/*
 * Copyright (C) 2015 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Kyle Nitzsche <kyle.nitzsche@canonical.com>
 *              Scott Sweeny <scott.sweeny@canonical.com>*
 */

#ifndef AGGSCOPEQUERY_H
#define AGGSCOPEQUERY_H

#include "aggchildscope.h"
#include "scope-ids.h"
#include "resultforwarder.h"

#include <functional>

#include <unity/scopes/SearchQueryBase.h>
#include <unity/scopes/ReplyProxyFwd.h>
#include <unity/scopes/CategorisedResult.h>
#include <unity-scopes.h>

class Query : public unity::scopes::SearchQueryBase
{
public:
    Query(const unity::scopes::CannedQuery &query,
                    const unity::scopes::SearchMetadata &metadata,
                    const std::string scope_dir,
                    const std::string cache_dir,
                    unity::scopes::RegistryProxy const& registry
                   );
    ~Query();
    virtual void cancelled() override;

    virtual void run(unity::scopes::SearchReplyProxy const& upstream_reply) override;

private:
    std::map<std::string,std::string> common_templates; // common_template id to common_template in json
    std::map<std::string,us::Category::SCPtr> registered_common_categories;//id is unique to scope and common category id.
    struct child_scope {
        std::string id;//required
        bool link_to_child = true;//when false, category has no link to child
        std::string category_title = "";//this is next required
        std::string category_title_ = "";//for i18n if present
        std::string surface_template = "";//required
        std::string search_template = "";//if not present, surface_template used for search
        int cardinality = 3;//optional
        std::string child_category = "";//optional
        int category_max_results = -1;//optional
        std::string child_department = "";//optional
        std::vector<std::shared_ptr<std::pair<std::string,std::string>>> swap_result_attributes;
        std::pair<std::string,std::string> if_category_use_template;
        std::map<std::string, std::string> result_category_id_to_common_template;//id is result category id. maps to id to use in common_templates
    };
    std::vector<std::shared_ptr<child_scope>> child_scopes;// pulled from child_sccopes.json
    std::map<std::string,std::shared_ptr<child_scope>> child_scopes_m;//id to child_scope
    void getChildScopes();

    std::vector<std::string> scope_order;// scope ids
    void make_child_scopes(std::string query_string);
    void handle_enabled_child_scopes(bool empty_search, us::SearchReplyProxy const& upstream_reply);
    void handle_child_scope_results(std::string query_string, bool empty_search, us::SearchReplyProxy const& upstream_reply);
    unity::scopes::RegistryProxy registry_;
    std::vector<us::ScopeProxy> scopes; // will hold all enabled and present child scopes
    std::map<std::string,std::shared_ptr<AggChildScope>> scopes_m;
    std::map<int,std::string> scopes_m_int_id;
    std::string scope_dir_;
    std::string cache_dir_;

    QString hints_is_hidden = "hints_is_hidden";
    bool check_hints();
    bool handle_hints(us::SearchReplyProxy const& upstream_reply_);
    bool is_hints_showing();
    void hide_hints();
};

#endif
