/*
 * Copyright (C) 2015 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Kyle Nitzsche <kyle.nitzsche@canonical.com>
 */

#include "aggchildscope.h"

namespace us = unity::scopes;

AggChildScope::AggChildScope(const std::string id, const std::string query_string_, bool use_category_link_to_child_) :
    id_(id),
    query_string(query_string_),
    using_category_link_to_child(use_category_link_to_child_)
{
    if (using_category_link_to_child)
        canned_query_ = std::make_shared<us::CannedQuery>(id, query_string, dept_id);
}

void AggChildScope::set_category_title(const std::string title_)
{
    cat_title = title_;
}

std::string AggChildScope::category_title()
{
    return cat_title;
}

void AggChildScope::set_category(us::Category::SCPtr category_)
{
    cat = category_;
}

us::Category::SCPtr AggChildScope::category()
{
    return cat;
}

void AggChildScope::set_surface_template(const std::string surface_template)
{
    TEMPLATE_SURFACE = surface_template;
}

std::string AggChildScope::surface_template()
{
    return TEMPLATE_SURFACE;
}

std::string AggChildScope::search_template()
{
    return TEMPLATE_SEARCH;
}

void AggChildScope::set_search_template(const std::string search_template)
{
    TEMPLATE_SEARCH = search_template;
}

void AggChildScope::set_child_department(const std::string department_id_)
{
    dept_id = department_id_;
    canned_query_->set_department_id(dept_id);
}

std::string AggChildScope::child_department()
{
    return dept_id;
}

void AggChildScope::set_child_category(const std::string category_id)
{
    category_id_ = category_id;
    using_category = true;
}

std::string AggChildScope::child_category()
{
    return category_id_;
}

void AggChildScope::set_category_max_results(int max)
{
    category_max_results_ = max;
    using_category_max_results_ = true;
}

bool AggChildScope::using_category_max_results()
{
    return using_category_max_results_;
}

int AggChildScope::category_max_results()
{
    return category_max_results_;
}

void AggChildScope::set_cardinality(int cardinality)
{
    cardinality_ = cardinality;
}

int AggChildScope::cardinality()
{
    return cardinality_;
}

std::string AggChildScope::id()
{
    return id_;
}

bool AggChildScope::category_link_to_child()
{
    return using_category_link_to_child;
}

std::shared_ptr<us::CannedQuery> AggChildScope::canned_query()
{
    return canned_query_;
}

void AggChildScope::set_proxy(us::RegistryProxy registry)
{
    auto scopes = registry->list();
    us::MetadataMap::const_iterator scope;
    scope = scopes.find(id_);
    if (scope != scopes.end()) {
        auto meta = registry->get_metadata(id_);
        proxy_ = meta.proxy();
        exists_ = true;
    }
}

us::ScopeProxy AggChildScope::proxy()
{
    return proxy_;
}

void AggChildScope::inc_result_idx()
{
    result_idx_++;
}

int AggChildScope::result_idx()
{
    return result_idx_;
}
