/*
 * Copyright (C) 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by Scott Sweeny <scott.sweeny@canonical.com>
 *
 */

#include"preview.h"

#include<unity/scopes/PreviewWidget.h>
#include<unity/scopes/ColumnLayout.h>
#include<unity/scopes/PreviewReply.h>

using namespace unity::scopes;

Preview::Preview(Result const& result, ActionMetadata const& metadata) :
    PreviewQueryBase(result, metadata)
{
}

Preview::~Preview()
{
}

void Preview::cancelled()
{
}

void Preview::run(unity::scopes::PreviewReplyProxy const& reply)
{
    PreviewWidgetList widgets;
    widgets.emplace_back(PreviewWidget(R"({"id": "header", "type": "header", "components" : { "title": "title", "subtitle": "author" } })"));
    widgets.emplace_back(PreviewWidget(R"({"id": "img", "type": "image", "components" : { "source": "screenshot-url" } })"));

    PreviewWidget w("img2", "image");
    w.add_attribute_value("zoomable", Variant(false));
    w.add_attribute_mapping("source", "screenshot-url");
    widgets.emplace_back(w);

    ColumnLayout layout1col(1);
    layout1col.add_column({"header", "img", "img2"});

    ColumnLayout layout2col(2);
    layout2col.add_column({"header", "img"});
    layout2col.add_column({"img2"});

    ColumnLayout layout3col(3);
    layout3col.add_column({"header"});
    layout3col.add_column({"img"});
    layout3col.add_column({"img2"});

    reply->register_layout({layout1col, layout2col, layout3col});
    reply->push(widgets);
    reply->push("author", Variant("Foo"));
    reply->push("screenshot-url", Variant("/path/to/image.png"));
}
